@@ -13,22 +13,21 @@ pub use crate::codegen::opcodes::UpvalueIndex;
 
 /// Call directive
 
-
 pub enum Call {
     Chunk(GC<Module>, FunctionID),
     Native(GC<NativeFunction>),
 }
 
-pub trait Invoke {
+pub trait Callable {
     fn signature(&self) -> &Signature;
-    fn as_call(&self) -> Call;
-    
-    fn invoke(&self, args: &[Variant]) -> ExecResult<Call> {
-        self.signature().check_args(args)?;
-        Ok(self.as_call())
-    }
-}
+    fn raw_call(&self, args: &[Variant]) -> Call;
 
+    fn checked_call(&self, args: &[Variant]) -> ExecResult<Call> {
+        self.signature().check_args(args)?;
+        Ok(self.raw_call(args))
+    }
+
+}
 
 // Compiled Functions
 
@@ -49,19 +48,30 @@ impl Function {
     pub fn proto(&self) -> &FunctionProto {
         self.module.data().get_function(self.fun_id)
     }
-}
-
-impl Invoke for Function {
-    fn signature(&self) -> &Signature {
+    
+    pub fn signature(&self) -> &Signature {
         self.proto().signature()
     }
+}
+
+impl Callable for Function {
+    fn signature(&self) -> &Signature { self.proto().signature() }
     
-    #[inline]
-    fn as_call(&self) -> Call {
+    fn raw_call(&self, _args: &[Variant]) -> Call {
         Call::Chunk(self.module, self.fun_id)
     }
 }
 
+impl Callable for GC<Function> {
+    fn signature(&self) -> &Signature {
+        <Function as Callable>::signature(self)
+    }
+    
+    fn raw_call(&self, args: &[Variant]) -> Call {
+        <Function as Callable>::raw_call(self, args)
+    }
+}
+
 unsafe impl GCTrace for Function {
     fn trace(&self) {
         self.module.mark_trace();
@@ -91,6 +101,12 @@ impl Closure {
     pub fn is_closed(&self) -> bool { matches!(self, Self::Closed(..)) }
 }
 
+unsafe impl GCTrace for Cell<Variant> {
+    fn trace(&self) {
+        self.get().trace()
+    }
+}
+
 
 #[derive(Debug, Clone)]
 pub struct Upvalue {
@@ -140,15 +156,15 @@ impl NativeFunction {
     }
     
     pub fn invoke(&self, args: &[Variant]) -> ExecResult<Variant> {
+        self.signature().check_args(args)?;
         (self.func)(self, args)
     }
 }
 
-impl Invoke for GC<NativeFunction> {
+impl Callable for GC<NativeFunction> {
     fn signature(&self) -> &Signature { &self.signature }
     
-    #[inline]
-    fn as_call(&self) -> Call {
+    fn raw_call(&self, _args: &[Variant]) -> Call {
         Call::Native(*self)
     }
 }
