@@ -1,6 +1,7 @@
 use std::fmt;
 use std::rc::Rc;
 use std::ops::Deref;
+use std::borrow::Borrow;
 use std::ptr::NonNull;
 use std::hash::{Hash, Hasher};
 use std::marker::PhantomData;
@@ -49,6 +50,11 @@ impl<T> GC<T> where T: GCTrace + ?Sized {
         self_gc.inner().ptr_eq(other_gc.inner())
     }
     
+    /// Casts the inner pointer to a usize. 
+    /// This is intended for identifying the GC, and should not be cast back to a pointer.
+    pub fn as_id(self_gc: &GC<T>) -> usize {
+        self_gc.ptr.as_ptr() as *const () as usize
+    }
 
 }
 
@@ -61,11 +67,17 @@ impl<T> From<GC<T>> for GC<dyn GCTrace> where T: GCTrace {
     }
 }
 
-// impl<T> From<Box<T>> for GC<T> where T: ?Sized {
-//     fn from(data: Box<T>) -> Self {
-//         unimplemented!()
-//     }
-// }
+impl<T> AsRef<T> for GC<T> where T: GCTrace + ?Sized {
+    fn as_ref(&self) -> &T {
+        self.deref()
+    }
+}
+
+impl<T> Borrow<T> for GC<T> where T: GCTrace + ?Sized {
+    fn borrow(&self) -> &T {
+        self.deref()
+    }
+}
 
 impl<T> Deref for GC<T> where T: GCTrace + ?Sized {
     type Target = T;
@@ -88,12 +100,11 @@ impl<T> Clone for GC<T> where T: GCTrace + ?Sized {
 impl<T> Copy for GC<T> where T: GCTrace + ?Sized { }
 
 impl<T> Hash for GC<T> where T: GCTrace {
-    fn hash<H>(&self, state: &mut H) where H: Hasher {
+    fn hash<H>(self: &Self, state: &mut H) where H: Hasher {
         <NonNull<GCBox<T>> as Hash>::hash(&self.ptr, state)
     }
 }
 
-
 impl<T> fmt::Debug for GC<T> where T: GCTrace + ?Sized {
     fn fmt(&self, fmt: &mut fmt::Formatter<'_>) -> fmt::Result {
         if fmt.alternate() {
