@@ -1,6 +1,8 @@
 use std::cell::Cell;
+use std::ops::Deref;
+use crate::codegen::LocalIndex;
 use crate::runtime::{Variant, HashMap};
-use crate::runtime::gc::GC;
+use crate::runtime::gc::{GC, GCTrace, gc_collect};
 use crate::runtime::function::{Call, Function, Upvalue, UpvalueIndex, Closure};
 use crate::runtime::module::Module;
 use crate::runtime::errors::ExecResult;
@@ -8,7 +10,6 @@ use crate::debug::traceback::TraceSite;
 use crate::debug::snapshot::{VMSnapshot, VMFrameSnapshot};
 
 mod callframe;
-pub use callframe::LocalIndex;
 
 use callframe::{VMCallFrame};
 
@@ -37,10 +38,12 @@ struct UpvalueRef {
     index: UpvalueIndex,
 }
 
-impl UpvalueRef {
+impl Deref for UpvalueRef {
+    type Target = Upvalue;
+    
     #[inline(always)]
-    fn borrow(&self) -> impl std::ops::Deref<Target=Upvalue> + '_ {
-        self.fun.get_upvalue(self.index)
+    fn deref(&self) -> &Self::Target {
+        &self.fun.upvalues()[usize::from(self.index)]
     }
 }
 
@@ -52,13 +55,6 @@ impl GC<Function> {
             index
         }
     }
-    
-    // Inserts an upvalue into a GC'd function, creating an UpvalueRef
-    #[inline(always)]
-    fn make_upvalue_ref(self, upvalue: Upvalue) -> UpvalueRef {
-        let idx = self.insert_upvalue(upvalue);
-        self.ref_upvalue(idx)
-    }
 }
 
 // struct UpvalueWeakRef
@@ -110,6 +106,8 @@ impl<'c> VirtualMachine<'c> {
             Control::Continue => { }
         }
         
+        gc_collect(self);
+        
         Ok(false)
     }
     
@@ -120,7 +118,7 @@ impl<'c> VirtualMachine<'c> {
             Call::Native(func) => {
                 let args = self.values.peek_many(call.nargs);
                 let retval = func.invoke(args)?;
-                self.values.truncate(call.frame.into());
+                self.values.truncate(call.frame);
                 self.values.push(retval);
                 self.traceback.pop();
             },
@@ -148,7 +146,7 @@ impl<'c> VirtualMachine<'c> {
         std::mem::swap(&mut self.frame, &mut frame);
         
         let retval = self.values.pop();
-        self.values.truncate(frame_idx.into());
+        self.values.truncate(frame_idx);
         self.values.push(retval);
         self.traceback.pop();
         
@@ -157,6 +155,32 @@ impl<'c> VirtualMachine<'c> {
     }
 }
 
+// trace through all GC roots
+unsafe impl GCTrace for VirtualMachine<'_> {
+    fn trace(&self) {
+        // trace through the value stack
+        for value in self.values.stack.iter() {
+            value.trace();
+        }
+        
+        // trace through the active frame and all calls
+        self.frame.trace();
+        for frame in self.calls.iter() {
+            frame.trace();
+        }
+        
+        // traceback
+        for site in self.traceback.iter() {
+            site.trace();
+        }
+        
+        // open upvalues
+        for upval_ref in self.upvalues.iter_refs() {
+            upval_ref.fun.mark_trace();
+        }
+    }
+}
+
 
 // Stack Manipulation
 #[derive(Debug)]
@@ -266,7 +290,7 @@ impl ValueStack {
     #[inline]
     fn get_closure(&self, closure: &Closure) -> Variant {
         match closure {
-            Closure::Open(index) => self.peek_at(*index).clone(),
+            Closure::Open(index) => *self.peek_at(*index),
             Closure::Closed(cell) => cell.get(),
         }
     }
@@ -294,24 +318,37 @@ impl OpenUpvalues {
         }
     }
     
+    fn iter_refs(&self) -> impl Iterator<Item=&UpvalueRef> {
+        self.upvalues.values().flat_map(|refs| refs.iter())
+    }
+    
+    fn register(&mut self, function: GC<Function>) {
+        for (index, upval) in function.upvalues().iter().enumerate() {
+            if upval.closure().is_open() {
+                let upval_ref = function.ref_upvalue(index.try_into().unwrap());
+                self.insert_ref(upval_ref);
+            }
+        }
+    }
+    
     fn insert_ref(&mut self, upval_ref: UpvalueRef) {
         // let weak_ref = upval_ref.into()  // TODO use weak references
         
-        let index = match upval_ref.borrow().value() {
+        let index = match upval_ref.closure() {
             Closure::Open(index) => index,
             _ => panic!("insert non-open upvalue"),
         };
         
         self.upvalues.entry(index)
             .and_modify(|refs| refs.push(upval_ref))
-            .or_insert(vec![ upval_ref ]);
+            .or_insert_with(|| vec![ upval_ref ]);
     }
     
     fn close_upvalues(&mut self, index: usize, value: Variant) {
         if let Some(upvalues) = self.upvalues.remove(&index) {
-            let gc_cell = GC::allocate(Cell::new(value));
+            let gc_cell = GC::new(Cell::new(value));
             for upvalue in upvalues.iter() {
-                upvalue.borrow().close(gc_cell)
+                upvalue.close(gc_cell)
             }
         }
     }
